/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;

import java.io.IOException;

public class Comm implements MessageListener {
    public static final int ACK_OK = 1;
    public static final int ACK_RESEND = 2;
    public static final int ACK_HAVEIT = 3;
    
    private PhoenixSource phoenix;
    private MoteIF moteIF;
    private CommGui commGui;
    private int checksum;
    private int missingLen;
    private String recvMsg;
    private String sendCode;
    private int len;
    private int destAddr;
    private short[] header = new short[28];
    private int counter = 0;
    private int recvMsgMissingLen;
    private int recvMsgChecksum;
    private boolean isSendingAgent;
    
    public Comm(CommGui cg, String source) {
        commGui = cg;
        
        if (source == null) {
            phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
        }
        else {
            phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
        }
        
        moteIF = new MoteIF(phoenix);
        moteIF.registerListener(new RadioClassMessage(), this);
        moteIF.registerListener(new RadioClassAck(), this);
    }
    
    public void messageReceived(int to, Message message) {
        int recvChecksum = 0;
        int code = ACK_OK;
        
        if (message.amType() == RadioClassMessage.AM_TYPE) { // zprava typu Message
            RadioClassMessage msg = (RadioClassMessage)message;
            short[] tmp = msg.get_msg();
            if (tmp[0] == 0)
            {
                recvMsg = "";
                recvMsgMissingLen = tmp[1]*256+tmp[2];
                recvMsgChecksum = tmp[3]*256+tmp[4];
                //System.out.println("Hlavicka: delka=" + recvMsgMissingLen + ", checksum=" + recvMsgChecksum);
            }
            else
            {
                for (int i = 1; i < 28; i++)
                {
                    if (recvMsgMissingLen <=0)
                    {
                        for (int j = 0;j < recvMsg.length(); j++) {
                            recvChecksum += recvMsg.charAt(j);
                        }
                        if (recvChecksum != recvMsgChecksum) {
                            System.err.println("Spatny checksum!");
                            code = ACK_RESEND;
                        } else {
                            System.out.println(message.getSerialPacket().get_header_src()+"->"+message.getSerialPacket().get_header_dest() + ": " + recvMsg);
                            commGui.addMsg(recvMsg, message.getSerialPacket().get_header_src()); // pridani do seznamu prijatych zprav v gui
                            break;
                        }
                    }
                    //System.out.println("Znak=" + (char)tmp[i]);
                    recvMsg += (char)tmp[i];
                    recvMsgMissingLen--;
                }
            }
            //System.out.println(message.getSerialPacket().get_header_src()+"->"+message.getSerialPacket().get_header_dest() + ": Received packet sequence number " + tmp[0]);
            sendAck(message.getSerialPacket().get_header_src(), code);
            //System.out.println(msg.toString());
        }
        else if (message.amType() == RadioClassAck.AM_TYPE) { // zprava typu ack
            RadioClassAck msg = (RadioClassAck)message;
            short[] tmp = msg.get_msg();
            //System.out.println("ACK recieved = " + tmp[0]);
            if (tmp[0] == ACK_OK || tmp[0] == ACK_HAVEIT) {
                counter++;
            } else if (tmp[0] == ACK_RESEND) {
                counter = 0;
                if (isSendingAgent) {
                    commGui.getInfoLabel().setText("Chyba v checksum, znovuposlání agenta!");
                } else {
                    commGui.getMsgInfoLabel().setText("Chyba v checksum, znovuposlání zprávy!");
                }
            }
            if (isSendingAgent) {
                sendPart();
            } else {
                sendMsgPart();
            }
        }
    }
    
    private void sendAck(int mote, int code) {
        RadioClassAck payload = new RadioClassAck();
        try
        {
            short[] tmp = new short[28];
            tmp[0] = (short)code;
            payload.set_msg(tmp);
            //System.out.println("Sending ACK packet");
            moteIF.send(mote, payload);
        }
        catch (IOException exception) {
            System.err.println("Exception thrown when sending packets. Exiting.");
            System.err.println(exception);
        }
    }
    
    private int min(int a, int b)
    {
        if (a<b)
            return a;
        else
            return b;
    }
  
    private void sendPart() {
        RadioClassAgent payload = new RadioClassAgent();
        
        try
        {
            if (counter == 0)
            {
                payload.set_msg(header);
                //System.out.println(payload.toString());
                moteIF.send(destAddr, payload);
                missingLen = len;
                //System.out.println("Sending AGENT header packet");
            }
            else
            {
                if (missingLen <= 0)
                {
                    //System.out.println("----------------------------");
                    //System.out.println("Agent sending was successful");
                    commGui.getInfoLabel().setText("Agent sending was successful!");
                    commGui.enableSendButtons();
                    return;
                }
                short[] tmp = new short[28];
                tmp[0] = (short)counter;
                for (int i = 1; i<=min(27,missingLen);i++)
                {
                    int index = i+ (counter-1)*27;
                    //System.out.println("Index: " + index);
                    tmp[i] = (short)sendCode.charAt(index-1);
                }
                missingLen -= 27;
                payload.set_msg(tmp);
                //System.out.println(payload.toString());
                //System.out.println("Sending AGENT packet: " + counter + missing_len);
                moteIF.send(destAddr, payload);
            }
        }
        catch (IOException exception) {
            System.err.println("Exception thrown when sending packets. Exiting.");
            System.err.println(exception);
        }
    }
    
    private void sendMsgPart() {
        RadioClassMessage payload = new RadioClassMessage();
        
        try
        {
            if (counter == 0)
            {
                payload.set_msg(header);
                //System.out.println(payload.toString());
                moteIF.send(destAddr, payload);
                missingLen = len;
                //System.out.println("Sending AGENT header packet");
            }
            else
            {
                if (missingLen <= 0)
                {
                    //System.out.println("----------------------------");
                    //System.out.println("Agent sending was successful");
                    commGui.getMsgInfoLabel().setText("Message sending was successful!");
                    commGui.enableSendButtons();
                    return;
                }
                short[] tmp = new short[28];
                tmp[0] = (short)counter;
                for (int i = 1; i<=min(27,missingLen);i++)
                {
                    int index = i+ (counter-1)*27;
                    //System.out.println("Index: " + index);
                    tmp[i] = (short)sendCode.charAt(index-1);
                }
                missingLen -= 27;
                payload.set_msg(tmp);
                //System.out.println(payload.toString());
                //System.out.println("Sending AGENT packet: " + counter + missing_len);
                moteIF.send(destAddr, payload);
            }
        }
        catch (IOException exception) {
            System.err.println("Exception thrown when sending packets. Exiting.");
            System.err.println(exception);
        }
    }
    
    public void sendAgent(int addr, String code, int pb, int plan, int bb, int ib, int reg1, int reg2, int reg3) {
        destAddr = addr;
        sendCode = new String(code);
        len = sendCode.length();
        missingLen = len;
        counter = 0;
        isSendingAgent = true;
        
        header[0] = 0;
        header[1] = (short)(len/256); // delka celeho kodu
        header[2] = (short)(len%256);
        
        checksum = 0;
        
        for (int i = 0; i<len; i++)
        {
            checksum += sendCode.charAt(i);
        }
        header[3] = (short)(checksum/256); // checksum
        header[4] = (short)(checksum%256);
        
        header[7] = (short)(pb/256); // planbase
        header[8] = (short)(pb%256);
        
        header[5] = (short)(plan/256); // plan
        header[6] = (short)(plan%256);
        
        header[9] = (short)(bb/256); // beliefbase
        header[10] = (short)(bb%256);
        
        header[11] = (short)(ib/256); // inputbase
        header[12] = (short)(ib%256);
        
        header[13] = (short)(reg1/256); // reg1
        header[14] = (short)(reg1%256);
        header[15] = (short)(reg2/256); // reg2
        header[16] = (short)(reg2%256);
        header[17] = (short)(reg3/256); // reg3
        header[18] = (short)(reg3%256);
        
        sendPart();
    }
    
    public void sendMsg(int addr, String msg) {
        sendCode = new String(msg);
        destAddr = addr;
        len = sendCode.length();
        missingLen = len;
        isSendingAgent = false;
        counter = 0;
        
        header[0] = 0;
        header[1] = (short)(len/256); // delka celeho kodu
        header[2] = (short)(len%256);
        
        checksum = 0;
        
        for (int i = 0; i<len; i++)
        {
            checksum += sendCode.charAt(i);
        }
        header[3] = (short)(checksum/256); // checksum
        header[4] = (short)(checksum%256);
        
        //System.out.println(msg+" --- "+ len + ": " + checksum);
        sendMsgPart();
    }
}
