/*									tab:4
 * "Copyright (c) 2005 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright
 * notice, the following two paragraphs and the author appear in all
 * copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/**
 * Java-side application for testing serial port communication.
 * 
 *
 * @author Phil Levis <pal@cs.berkeley.edu>
 * @date August 12 2005
 */

import java.io.IOException;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;

import java.io.*;

public class SendAgent implements MessageListener {

  private MoteIF moteIF;
  private int msg_len;
  private int can_run = 1;
  //private int checksum;
  private int missing_len;
  private static String posilany_kod;
  private static int delka_kodu;
  private static int dest_addr;
  //private String kod;
  private static short[] hlavicka = new short[28];
  private static int counter = 0;
  
  public SendAgent(MoteIF moteIF) {
    this.moteIF = moteIF;
    this.moteIF.registerListener(new RadioClassAck(), this);
  }
  
  public int min(int a, int b)
  {
    if (a<b)
      return a;
    else
      return b;
  }
  
  public void sendPart() {
    RadioClassAgent payload = new RadioClassAgent();
    try
    {
      if (counter == 0)
      {
        payload.set_msg(hlavicka);
        moteIF.send(dest_addr, payload);
        missing_len = delka_kodu;
        //System.out.println("Sending AGENT header packet");
      }
      else
      {
        if (missing_len <= 0)
        {
          System.out.println("----------------------------");
          System.out.println("Agent sending was successful");
          System.exit(0);
        }
        short[] tmp = new short[28];
        tmp[0] = (short)counter;
        for (int i = 1; i<min(28,missing_len);i++)
        {
          int index = i+ (counter-1)*27;
          //System.out.println("Index: " + index);
          if (index == delka_kodu)
          {
            tmp[i] = 0;
          }
          else
          {
            tmp[i] = (short)posilany_kod.charAt(index-1);
          }
        }
        missing_len -= 27;
        payload.set_msg(tmp);
        //System.out.println("Sending AGENT packet: " + counter + missing_len);
        moteIF.send(2, payload);
      }
    }
    catch (IOException exception) {
      System.err.println("Exception thrown when sending packets. Exiting.");
      System.err.println(exception);
    }
  }

  public void messageReceived(int to, Message message) {
    RadioClassAck msg = (RadioClassAck)message;
    short[] tmp = msg.get_msg();
    //System.out.println("ACK recieved = " + tmp[0]);
    if (tmp[0] == 1)
    {
      counter++;
      sendPart();
    }

  }
  
  private static void usage() {
    System.err.println("usage: TestSerial <file> <moteID> -comm <source>");
  }
  
  private static void nacti_kod(BufferedReader br)
  {
    posilany_kod = "";
    String strLine;
    String strB = "";
    String strD = "";
    String strI = "";
    String stri = "";
    String str1 = "";
    String str2 = "";
    String str3 = "";
    try
    {
      while ((strLine = br.readLine()) != null)
      {
        if (strLine.charAt(0) == 'B')
        {
          strB += strLine.substring(1);
        }
        else if (strLine.charAt(0) == 'D')
        {
          strD += strLine.substring(1);
        }
        else if (strLine.charAt(0) == 'I')
        {
          strI = strLine.substring(1);
        }
        else if (strLine.charAt(0) == 'i')
        {
          stri += strLine.substring(1);
        }
        else if (strLine.charAt(0) == '1')
        {
          str1 = strLine.substring(1);
        }
        else if (strLine.charAt(0) == '2')
        {
          str2 = strLine.substring(1);
        }
        else if (strLine.charAt(0) == '3')
        {
          str3 = strLine.substring(1);
        }
      }
      System.out.println ("Belief: "+strB);
      System.out.println ("Desire: "+strD);
      System.out.println ("Intention: "+strI);
      System.out.println ("InputBase: "+stri);
      System.out.println ("REG1: "+str1);
      System.out.println ("REG2: "+str2);
      System.out.println ("REG3: "+str3);
      posilany_kod = strD + strI + strB + stri + str1 + str2 + str3 + "\0";
      hlavicka[0] = 0;
      int len = posilany_kod.length()+1;
      delka_kodu = len;
      hlavicka[1] = (short)(len/256);
      hlavicka[2] = (short)(len%256);
      int checksum = 0;
      for (int i = 0; i<posilany_kod.length(); i++)
      {
        checksum += posilany_kod.charAt(i);
      }
      hlavicka[3] = (short)(checksum/256);
      hlavicka[4] = (short)(checksum%256);
      len = strD.length(); //plan - I
      hlavicka[5] = (short)(len/256);
      hlavicka[6] = (short)(len%256);
      len = 0; //plany - D
      hlavicka[7] = (short)(len/256);
      hlavicka[8] = (short)(len%256);
      len = strD.length() + strI.length(); //znalosti - B
      hlavicka[9] = (short)(len/256);
      hlavicka[10] = (short)(len%256);
      len = strD.length() + strI.length() + strB.length(); //vstupy - i
      hlavicka[11] = (short)(len/256);
      hlavicka[12] = (short)(len%256);
      len = strD.length() + strI.length() + strB.length() + stri.length(); //REG1
      hlavicka[13] = (short)(len/256);
      hlavicka[14] = (short)(len%256);
      len = strD.length() + strI.length() + strB.length() + stri.length() + str1.length(); //REG1
      hlavicka[15] = (short)(len/256);
      hlavicka[16] = (short)(len%256);
      len = strD.length() + strI.length() + strB.length() + stri.length()+ str1.length() + str2.length(); //REG1
      hlavicka[17] = (short)(len/256);
      hlavicka[18] = (short)(len%256);
      
      System.out.println ("posilany_kod: "+posilany_kod);
      System.out.println ("checksum: "+checksum);
      //System.out.println ("hlavicka: "+hlavicka);
    }
    catch (Exception e)
    {
      System.err.println("Error: " + e.getMessage());
    }
  }
  
  public static void main(String[] args) throws Exception {
    String source = null;
    if (args.length == 4) {
      if (!args[2].equals("-comm")) {
        usage();
        System.exit(1);
      }
      dest_addr = Integer.valueOf( args[1] ).intValue();
      System.out.println ("Cilova adresa: " + dest_addr);
      FileInputStream fstream = new FileInputStream(args[0]);
      DataInputStream in = new DataInputStream(fstream);
      BufferedReader br = new BufferedReader(new InputStreamReader(in));
      nacti_kod(br);
      in.close();
      source = args[3];
      //return;
    }
    else if (args.length != 0) {
      usage();
      System.exit(1);
    }

    PhoenixSource phoenix;
    
    if (source == null) {
      phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
    }
    else {
      phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
    }

    MoteIF mif = new MoteIF(phoenix);
    SendAgent serial = new SendAgent(mif);
    serial.sendPart();
  }


}

