/*									tab:4
 * "Copyright (c) 2005 The Regents of the University  of California.  
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose, without fee, and without written
 * agreement is hereby granted, provided that the above copyright
 * notice, the following two paragraphs and the author appear in all
 * copies of this software.
 * 
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY
 * PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE UNIVERSITY OF CALIFORNIA HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE UNIVERSITY OF CALIFORNIA HAS NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS."
 *
 */

/**
 * Java-side application for testing serial port communication.
 * 
 *
 * @author Phil Levis <pal@cs.berkeley.edu>
 * @date August 12 2005
 */

import java.io.IOException;

import net.tinyos.message.*;
import net.tinyos.packet.*;
import net.tinyos.util.*;

public class TestSerial implements MessageListener {

  private MoteIF moteIF;
  private int msg_len;
  private int checksum;
  private int missing_len;
  private String kod;
  
  public TestSerial(MoteIF moteIF) {
    this.moteIF = moteIF;
    this.moteIF.registerListener(new RadioClassMessage(), this);
  }
  
  public void sendAck(int mote) {
    RadioClassAck payload = new RadioClassAck();
    try
    {
      short[] tmp = new short[28];
      tmp[0] = 1; //ACK_OK
      payload.set_msg(tmp);
      //System.out.println("Sending ACK packet");
      moteIF.send(mote, payload);
    }
    catch (IOException exception) {
      System.err.println("Exception thrown when sending packets. Exiting.");
      System.err.println(exception);
    }
  }

  public void messageReceived(int to, Message message) {
    RadioClassMessage msg = (RadioClassMessage)message;
    short[] tmp = msg.get_msg();
    if (tmp[0] == 0)
    {
      kod = "";
      msg_len = tmp[1]*256+tmp[2];
      missing_len = msg_len;
      checksum = tmp[3]*256+tmp[4];
      kod = "";
      //System.out.println("Hlavicka: delka=" + msg_len + ", checksum=" + checksum);
    }
    else
    {
      for (int i = 1; i < 28; i++)
      {
        if (missing_len <=0)
        {
          System.out.println(message.getSerialPacket().get_header_src()+"->"+message.getSerialPacket().get_header_dest() + ": " + kod.toString());
          break;
        }
        //System.out.println("Znak=" + (char)tmp[i]);
        kod = kod + (char)tmp[i];
        missing_len--;
      }
    }
//     message.getSerialPacket().get_header_src()
    //System.out.println(message.getSerialPacket().get_header_src()+"->"+message.getSerialPacket().get_header_dest() + ": Received packet sequence number " + tmp[0]);
    sendAck(message.getSerialPacket().get_header_src());
    //System.out.println(msg.toString());
  }
  
  private static void usage() {
    System.err.println("usage: TestSerial [-comm <source>]");
  }
  
  public static void main(String[] args) throws Exception {
    String source = null;
    if (args.length == 2) {
      if (!args[0].equals("-comm")) {
	usage();
	System.exit(1);
      }
      source = args[1];
    }
    else if (args.length != 0) {
      usage();
      System.exit(1);
    }
    
    PhoenixSource phoenix;
    
    if (source == null) {
      phoenix = BuildSource.makePhoenix(PrintStreamMessenger.err);
    }
    else {
      phoenix = BuildSource.makePhoenix(source, PrintStreamMessenger.err);
    }

    MoteIF mif = new MoteIF(phoenix);
    TestSerial serial = new TestSerial(mif);
    //serial.sendPackets();
  }


}
